/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener;

import static org.ow2.petals.binding.soap.Constants.ServiceUnit.REMOVE_ROOT;
import static org.ow2.petals.component.framework.Constants.Component.TIMEOUT;

import java.util.Properties;



/**
 * @author chamerling - eBM Websourcing
 *
 */
public class SUPropertiesHelper {

    /**
     * Creates a new instance if {@link SUPropertiesHelper}
     */
    protected SUPropertiesHelper() {
        super();
    }

    /**
     * Retrieve the timeout element extension value
     *
     * @param extensionProperties
     * @return The timeout value set in extensions. The default value is
     *         returned if no valid value is set or if no value is set.
     */
    public final static long retrieveTimeOut(
            final Properties extensionProperties, final long defaultValue) {

        final String tmp = extensionProperties
                .getProperty(TIMEOUT);
        long result = defaultValue;

        try {
            result = (tmp == null ? defaultValue : Long.parseLong(tmp));
        } catch (NumberFormatException e) {
            // NOP
        }

        return result;
    }

    /**
     * Retrieve the remove root element extension value
     *
     * @param extensionProperties
     * @return
     */
    public final static boolean retrieveRemoveRootElement(
            final Properties extensionProperties, final boolean defaultValue) {

        final String tmp = extensionProperties
                .getProperty(REMOVE_ROOT);

        return tmp == null ? defaultValue : Boolean.parseBoolean(tmp);
    }
}
