/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.binding.soap.listener.outgoing;

import static org.ow2.petals.binding.soap.Constants.ServiceUnit.ADDRESS;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.util.XMLUtils;
import org.ow2.petals.binding.soap.SoapComponentContext;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.ws.notification.WsnManager;
import org.w3c.dom.Document;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
public class WSNotifier extends AbstractJBIDispatcher {

    /**
     * Creates a new instance of {@link WSNotifier}
     * 
     * @param soapContext
     * @param logger
     */
    public WSNotifier(SoapComponentContext soapContext, Logger logger) {
        super(soapContext, logger);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.binding.soap.listener.outgoing.JBIListenerDispatcher#process(org.objectweb.petals.component.common.util.MessageExchangeWrapper,
     *      org.objectweb.petals.common.descriptor.Extensions)
     */
    public void process(Exchange exchange, Extensions cdkExtensions) {
        QName topicName = new QName(cdkExtensions.getPetalsExtensions().getProperty(ADDRESS));
        logger.fine("Posting message to topic : " + topicName);

        if (!"".equals(topicName)) {
            sendNotification(topicName, exchange);
        } else {
            String message = "No topic name has been found, the message can not be delivered";
            this.logger.warning(message);
            try {
                exchange.setFault(new Exception(message));
            } catch (MessagingException e1) {
                logger.log(Level.SEVERE, "Can't return fault to consumer", e1);
            }
        }
    }

    /**
     * Send a notification to all the topic subscribers
     * 
     * @param topicName
     * @param exchange
     */
    protected void sendNotification(QName topicName, Exchange exchange) {

        WsnManager manager = soapContext.getWsnManager();

        try {
            OMElement notification = createNotificationContentFromJBIMessage(exchange);
            manager.publish(topicName, notification, new Axis2NotificationConsumerClient(
                    this.logger));
        } catch (Exception e) {
            try {
                exchange.setFault(e);
            } catch (MessagingException e1) {
                logger.log(Level.SEVERE, "Can't return fault to consumer", e1);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.binding.soap.listener.outgoing.JBIListenerDispatcher#getDispatcherType()
     */
    public String getDispatcherType() {
        return "TOPIC";
    }

    /**
     * Create the notification message from the JBI message exchange
     * 
     * @param exchange
     * @return
     */
    public OMElement createNotificationContentFromJBIMessage(Exchange exchange) throws Exception {
        final Document contentDocument = exchange.getInMessageContentAsDocument();
        return XMLUtils.toOM(contentDocument.getDocumentElement());
    }
}
